package net.minecraft.src.forestry.api;

import java.util.ArrayList;
import java.util.HashMap;

import net.minecraft.src.Block;
import net.minecraft.src.Item;
import net.minecraft.src.ItemStack;

/**
 * Simple API class to add your own resources to Forestry
 * If you need something added/changed/fixed, drop me a PM on the Minecraft forums and I'll see what can be done.
 * @author SirSengir
 */
public class APIBridge {

	/**
	 * Allows you to add recipes to the carpenter. See {@link ICarpenterManager} for details.
	 */
	public static ICarpenterManager carpenterManager;
	/**
	 * Add new fuels for EngineBronze (= biogas engine) here
	 */
	public static HashMap<Integer, EngineBronzeFuel> bronzeEngineFuel = new HashMap<Integer, EngineBronzeFuel>();
	/**
	 * Add new fuels for EngineCopper (= peat-fired engine) here
	 */
	public static HashMap<Integer, EngineCopperFuel> copperEngineFuel = new HashMap<Integer, EngineCopperFuel>();
	/**
	 * Add new fuels for the fermenter here (i.e. fertilizer)
	 */
	public static HashMap<Integer, FermenterFuel> fermenterFuel = new HashMap<Integer, FermenterFuel>();
	/**
	 * Add new resources for the fermenter here (i.e saplings)
	 */
	public static HashMap<Integer, FermenterResource> fermenterResource = new HashMap<Integer, FermenterResource>();
	/**
	 * Add new resources for the moistener here (i.e. wheat)
	 */
	public static HashMap<Integer, MoistenerResource> moistenerResource = new HashMap<Integer, MoistenerResource>();
	
	/**
	 * Register an item as wrench. Items with the passed itemid can be used to rotate {@BlockEngine}s. Not metadata aware!
	 * @param itemid
	 * @return
	 */
	public static boolean registerWrench(int itemid) {
		if(wrenchIds.contains(itemid))
			return false;
		
		wrenchIds.add(itemid);
		return true;
	}
	
	/**
	 * Registers a {@link LiquidContainer} (bucket or bucket replacement) for use as a lava container in Forestry's machines. 
	 * @param container
	 * @return
	 */
	public static boolean registerLavaContainer(LiquidContainer container) {
		// Key already taken, skip adding this container
		if(lavaContainersByFullId.containsKey(container.filled.itemID))
			return false;
		
		lavaContainersByFullId.put(container.filled.itemID, container);
		if(container.empty != null)
			lavaContainersByEmptyId.put(container.empty.itemID, container);
		return true;
	}
	
	/**
	 * Registers a {@link LiquidContainer} (bucket or bucket replacement) for use as a water container in Forestry's machines. 
	 * @param container
	 * @return
	 */
	public static boolean registerWaterContainer(LiquidContainer container) {
		// Key already taken, skip adding this container
		if(waterContainersByFullId.containsKey(container.filled.itemID))
			return false;
		
		waterContainersByFullId.put(container.filled.itemID, container);
		if(container.empty != null)
			waterContainersByEmptyId.put(container.empty.itemID, container);
		return true;
	}
	
	/**
	 * Registers a {@link LiquidContainer} (bucket or bucket replacement) for use as a biomass container in Forestry's machines. 
	 * @param container
	 * @return
	 */
	public static boolean registerBiomassContainer(LiquidContainer container) {
		// Key already taken, skip adding this container
		if(biomassContainersByFullId.containsKey(container.filled.itemID))
			return false;
		
		biomassContainersByFullId.put(container.filled.itemID, container);
		if(container.empty != null)
			biomassContainersByEmptyId.put(container.empty.itemID, container);
		return true;
	}
	
	/**
	 * Registers a {@link LiquidContainer} (bucket or bucket replacement) for use as a biofuel container in Forestry's machines. 
	 * @param container
	 * @return
	 */
	public static boolean registerBiofuelContainer(LiquidContainer container) {
		// Key already taken, skip adding this container
		if(biofuelContainersByFullId.containsKey(container.filled.itemID))
			return false;
		
		biofuelContainersByFullId.put(container.filled.itemID, container);
		if(container.empty != null)
			biofuelContainersByEmptyId.put(container.empty.itemID, container);
		return true;
	}
	
	// EVERYTHING BELOW IS FOR INTERNAL USE ONLY - SHOULD BE SELF EXPLANATORY	
	private static ArrayList<Integer> wrenchIds = new ArrayList<Integer>();	
	private static HashMap<Integer, LiquidContainer> lavaContainersByFullId = new HashMap<Integer, LiquidContainer>();
	private static HashMap<Integer, LiquidContainer> lavaContainersByEmptyId = new HashMap<Integer, LiquidContainer>();
	private static HashMap<Integer, LiquidContainer> waterContainersByFullId = new HashMap<Integer, LiquidContainer>();
	private static HashMap<Integer, LiquidContainer> waterContainersByEmptyId = new HashMap<Integer, LiquidContainer>();
	private static HashMap<Integer, LiquidContainer> biomassContainersByFullId = new HashMap<Integer, LiquidContainer>();
	private static HashMap<Integer, LiquidContainer> biomassContainersByEmptyId = new HashMap<Integer, LiquidContainer>();
	private static HashMap<Integer, LiquidContainer> biofuelContainersByFullId = new HashMap<Integer, LiquidContainer>();
	private static HashMap<Integer, LiquidContainer> biofuelContainersByEmptyId = new HashMap<Integer, LiquidContainer>();
	
	public static boolean hasLavaContainerByFullId(int id) {
		return lavaContainersByFullId.containsKey(id);
	}
	public static boolean hasLavaContainerByEmptyId(int id) {
		return lavaContainersByEmptyId.containsKey(id);
	}
	public static LiquidContainer getLavaContainerByFullId(int id) {
		return lavaContainersByFullId.get(id);
	}
	public static LiquidContainer getLavaContainerByEmptyId(int id) {
		return lavaContainersByEmptyId.get(id);
	}
	
	public static boolean hasWaterContainerByFullId(int id) {
		return waterContainersByFullId.containsKey(id);
	}
	public static boolean hasWaterContainerByEmptyId(int id) {
		return waterContainersByEmptyId.containsKey(id);
	}
	public static LiquidContainer getWaterContainerByFullId(int id) {
		return waterContainersByFullId.get(id);
	}
	public static LiquidContainer getWaterContainerByEmptyId(int id) {
		return waterContainersByEmptyId.get(id);
	}
	
	public static boolean hasBiomassContainerByFullId(int id) {
		return biomassContainersByFullId.containsKey(id);
	}
	public static boolean hasBiomassContainerByEmptyId(int id) {
		return biomassContainersByEmptyId.containsKey(id);
	}
	public static LiquidContainer getBiomassContainerByFullId(int id) {
		return biomassContainersByFullId.get(id);
	}
	public static LiquidContainer getBiomassContainerByEmptyId(int id) {
		return biomassContainersByEmptyId.get(id);
	}
	
	public static boolean hasBiofuelContainerByFullId(int id) {
		return biofuelContainersByFullId.containsKey(id);
	}
	public static boolean hasBiofuelContainerByEmptyId(int id) {
		return biofuelContainersByEmptyId.containsKey(id);
	}
	public static LiquidContainer getBiofuelContainerByFullId(int id) {
		return biofuelContainersByFullId.get(id);
	}
	public static LiquidContainer getBiofuelContainerByEmptyId(int id) {
		return biofuelContainersByEmptyId.get(id);
	}
	
	public static boolean isWrench(ItemStack item) {
		return wrenchIds.contains(item.itemID);
	}
}
